#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_spline.h>

#define RES 300

int main (void)
{
	int N = 6;
	double x[6] = {0., 1., 2., 3., 4., 5. };
	double y[6] = {1., 2., 1., 2., 0., 0. };
	gsl_interp_accel *acc = gsl_interp_accel_alloc ();
	const gsl_interp_type *t = gsl_interp_akima;
	gsl_spline *spline = gsl_spline_alloc (t, N);

	int i; double xi, yi;

	/* for gnuplot */
	printf("set terminal epslatex size 10.5cm, 7.5cm\n");
	printf("set output \"interp2akima.tex\"\n");
	printf("set nokey\n");
	printf("plot [:][:] '-' with points,"	/* Given data x[], y[] */
	                   "'-' with dots,"     /* spline curve */
	                   "'-' with dots,"     /* derivative */
	                   "'-' with dots\n");  /* 2nd derivative */

	/* plot given data */
	for (i = 0; i < N; i++) printf ("%g %g\n", x[i], y[i]);
	printf("e\n");

	/* plot spline curve */
	gsl_spline_init (spline, x, y, N);
	for (i = 0; i <= RES; i++) {
	    xi = (1 - i / (double)RES) * x[0] + (i / (double)RES) * x[N-1];
	    yi = gsl_spline_eval (spline, xi, acc);
	    printf ("%g %g\n", xi, yi);
	}
	printf("e\n");

	/* plot the first derivative */
	for (i = 0; i <= RES; i++) {
	    xi = (1 - i / (double)RES) * x[0] + (i / (double)RES) * x[N-1];
	    yi = gsl_spline_eval_deriv (spline, xi, acc);
	    printf ("%g %g\n", xi, yi);
	}
	printf("e\n");

	/* plot the second derivative */
	for (i = 0; i <= RES; i++) {
	    xi = (1 - i / (double)RES) * x[0] + (i / (double)RES) * x[N-1];
	    yi = gsl_spline_eval_deriv2 (spline, xi, acc);
	    printf ("%g %g\n", xi, yi);
	}
	printf("e\n");

	gsl_spline_free (spline);
	gsl_interp_accel_free (acc);

	return 0;
}

