#include <stdio.h>
#include <gsl/gsl_multimin.h>

/* 中心が (dp[0],dp[1]) の放物面 */

double my_f (const gsl_vector *v, void *params)
{
    double x, y;
    double *dp = (double *)params;

    x = gsl_vector_get(v, 0);
    y = gsl_vector_get(v, 1);

    return 10.0 * (x - dp[0]) * (x - dp[0]) +
           20.0 * (y - dp[1]) * (y - dp[1]) + 30.0;
}

/* f の勾配 df = (df/dx, df/dy). */
void my_df (const gsl_vector *v, void *params, gsl_vector *df)
{
    double x, y;
    double *dp = (double *)params;

    x = gsl_vector_get(v, 0);
    y = gsl_vector_get(v, 1);
    gsl_vector_set(df, 0, 20.0 * (x - dp[0]));
    gsl_vector_set(df, 1, 40.0 * (y - dp[1]));
}

/* f と df の両方を計算 */
void my_fdf (const gsl_vector *x, void *params, double *f, gsl_vector *df)
{
    *f = my_f(x, params);
    my_df(x, params, df);
}

int main(void)
{
    size_t np = 2;
    double par[5] = {1.0, 2.0, 10.0, 20.0, 30.0};

    const gsl_multimin_fminimizer_type *T =
        gsl_multimin_fminimizer_nmsimplex;
    gsl_multimin_fminimizer *s = NULL;
    gsl_vector *ss, *x;
    gsl_multimin_function minex_func;

    size_t iter = 0, i;
    int status;
    double size;

    /* 探索開始点 */ 
    x = gsl_vector_alloc(2);
    gsl_vector_set(x, 0, 5.0);
    gsl_vector_set(x, 1, 7.0);

    /* ステップ幅の初期値は全て 1 */ 
    ss = gsl_vector_alloc(2);
    gsl_vector_set_all (ss, 1.0);

    /* インスタンスの初期化 */ 
    minex_func.n = 2;
    minex_func.f = &my_f;
    minex_func.params = (void *)&par;
    s = gsl_multimin_fminimizer_alloc(T, 2);

    gsl_multimin_fminimizer_set(s, &minex_func, x, ss);

    do {
        iter++;
        status = gsl_multimin_fminimizer_iterate(s);
        if (status) break;

        size = gsl_multimin_fminimizer_size(s);
        status = gsl_multimin_test_size (size, 1e-2);
        if (status == GSL_SUCCESS) printf("converged to minimum at\n"); 

        printf("%5d %.5f %.5f %10.5f\n", iter,
                                         gsl_vector_get (s->x, 0),
                                         gsl_vector_get (s->x, 1),
                                         s->f);
    } while (status == GSL_CONTINUE && iter < 100);

    gsl_vector_free(x);
    gsl_vector_free(ss);
    gsl_multimin_fminimizer_free(s);

    return status;
} 
